# coding=utf-8
from .datastore import DataStore
from .datastore import open_datastore
from .datastore import read_xml_dir

__version__ = '0.1.2'
__all__ = ["DataStore", "open_datastore", "read_xml_dir"]
